/** 
 * This is a JDateField that extends the JTextField and uses the DateFieldDocument.
 *
 * @author Frank Gehde
 * @version 1.2
 */

//Besitzt Zugriffsmethoden fr die Datumsfelder.
//Copy and Paste geht nur mit Einzel Zeichen oder dem ganzen Datum. Nicht 
//mit Einzelfeldern (Ursache: Document).

import java.util.*;		// Fuer Calendar Objekte
import javax.swing.*;		// Weil JTextField erweitert wird

public class JDateField extends JTextField {	// **** Beginn Klasse JDateField

	//**** zustzliche Attribute
	private Calendar myDates = new GregorianCalendar();	// Calender fuers init.
	private int tagInt = myDates.get(Calendar.DATE);	
	private int monatInt = myDates.get(Calendar.DATE);	// Felder, die Teilstrings
	private int jahrInt = myDates.get(Calendar.DATE);	// des gesamten TextFeldes
	private String tagString = String.valueOf(tagInt);	// enthalten, bzw. die ent-
	private String monatString = String.valueOf(monatInt);	// sprechenden Int Werte dazu
	private String jahrString = String.valueOf(jahrInt);
	// zur Datumsgltigkeit:
	private static final int[] tageMax = {31,28,31,30,31,30,31,31,30,31,30,31}; 
	//**** zustzliche Attribute Ende

	//****** Konstruktoren ******
        // Konstruktor initialisiert mit Datum = HEUTE
	public JDateField() {					// Kein Para (Def. Datum: Heute)
		super();					// Wie gehabt
		this.setDocument(new DateFieldDocument(this));  // Passendes Dokument
	}

        // Konstruktor spezifiziert Anzahl Spalten und heutiges Datum
	public JDateField(int columns) {			// Breite (Default Datum: Heute)
		super(columns);					// Wie gehabt
		this.setDocument(new DateFieldDocument(this));  // Passendes Dokument
	}

        // Konstruktor fr einen Datums-String der Form "tt.mm.jjjj"
	public JDateField(String text) {		// Datum als String der Form xx.xx.xxxx
		super(text);				// Wie gehabt
		this.setDocument(new DateFieldDocument(this));  // Passendes Dokument
	}

        // Konstruktor fr einen Datums-String der Form "tt.mm.jjjj" 
	// und die Anzahl der Spalten 
	public JDateField(String text, int columns) {		// Kombination aus 2 und 3
		super(text, columns);				// Wie gehabt
		this.setDocument(new DateFieldDocument(this));  // Passendes Dokument
	}

        // Konstruktor der mit einem Calendar Objekt initialisiert wird
	public JDateField(Calendar myDate) {			// SpezialConstructor des Documents
		super();					// Wie gehabt
		this.setDocument(new DateFieldDocument(this, myDate));  // Passendes Dokument
	}

        // Konstruktor der mit einem Calendar Objekt initialisiert wird
	// und der Anzahl der Spalten
	public JDateField(Calendar myDate, int columns) {	// SpezialConstructor des Documents
		super(columns);					// Wie gehabt
		this.setDocument(new DateFieldDocument(this, myDate));  // Passendes Dokument
	}

        // Konstruktor mit drei int-Werten fr Tag, Monat und Jahr 
	public JDateField(int tag, int monat, int jahr) {	// Mit Int Werten frs Datum
		super();					// Wie gehabt
		this.setDocument(new DateFieldDocument(this,	// Passendes Dokument
				     new GregorianCalendar(jahr, monat, tag)));
	}

	// Konstruktor mit drei int-Werten fr Tag, Monat und Jahr 
	// und einem int-Wert fr die Anzahl der Spalten
	public JDateField(int tag, int monat, int jahr, int columns) {	// Mit Int Werten frs
		super(columns);						// Datum und Spalten
		this.setDocument(new DateFieldDocument(this,	// Passendes Dokument
				     new GregorianCalendar(jahr, monat, tag)));
	}
	//****** Konstruktoren Ende *

	//****** SET METHODEN *******
							// **** set-Methoden fr ganze Daten
	public void setDate(Calendar myNewDate) {  	// setzt Datum nach bergabe eines Calender
		String myTagString=String.valueOf(myNewDate.get(Calendar.DATE));
		String myMonatString=String.valueOf(myNewDate.get(Calendar.MONTH)+1);
		String myJahrString=String.valueOf(myNewDate.get(Calendar.YEAR)); 
		this.setText(myTagString+"."+myMonatString+"."+myJahrString);
	}	
	public void setDate(String myDateText) {  	// setzt das Datum nach bergabe eines String
		//attributeUpdate();			// zB "11.12.2000"
		if(isDate(myDateText)==true)
			this.setText(myDateText);	// nicht robust !!
	}
	public void setDate(int day, int month, int year) {  	// setzt das Datum nach bergabe 
		String myTagString=String.valueOf(day);		// dreier int Werte 
		if(day<10) myTagString="0"+String.valueOf(day);
		String myMonatString=String.valueOf(month);
		if(month<10) myMonatString="0"+String.valueOf(month);
		String myJahrString=String.valueOf(year);
		if(year<10) myJahrString="000"+String.valueOf(year);
		if(year<100) myJahrString="00"+String.valueOf(year);
		if(year<1000) myJahrString="0"+String.valueOf(year);
		this.setText(myTagString+"."+myMonatString+"."+myJahrString);
	}

	public void setDay(int day) {  			// **** set-Methoden fr Int Werte
		attributeUpdate();			// setzt den Tag nach bergabe eines int 
		String myTagString=String.valueOf(day);
		if(day<10) myTagString="0"+String.valueOf(day);
		this.setText(myTagString+"."+monatString+"."+jahrString);
	}
	public void setMonth(int month) {  		// setzt den Monat nach bergabe eines int
		attributeUpdate();			// 
		String myMonatString=String.valueOf(month);
		if(month<10) myMonatString="0"+String.valueOf(month);
		this.setText(tagString+"."+myMonatString+"."+jahrString);
	}
	public void setYEAR(int year) {  		// setzt das Jahr nach bergabe eines int
		attributeUpdate();			// 
		String myJahrString=String.valueOf(year);
		if(year<10) myJahrString="000"+String.valueOf(year);
		if(year<100) myJahrString="00"+String.valueOf(year);
		if(year<1000) myJahrString="0"+String.valueOf(year);
		this.setText(tagString+"."+monatString+"."+myJahrString);
	}

	public void setDayText(String day) {  		// **** set-Methoden fr String Werte
		attributeUpdate();			// setzt den Tag nach bergabe eines String 
		String myTagString=day;
		if(day.length()<2) myTagString="0"+day;
		this.setText(myTagString+"."+monatString+"."+jahrString);
	}
	public void setMonthText(String month) {  	// setzt den Monat nach bergabe eines String
		attributeUpdate();			// 
		String myMonatString=month;
		if(month.length()<2) myMonatString="0"+month;
		this.setText(tagString+"."+myMonatString+"."+jahrString);
	}
	public void setYearText(String year) {  	// setzt das Jahr nach bergabe eines String
		attributeUpdate();			// 
		String myJahrString=year;
		if(year.length()==1) myJahrString="000"+year;
		if(year.length()==2) myJahrString="00"+year;
		if(year.length()==3) myJahrString="0"+year;
		this.setText(tagString+"."+monatString+"."+myJahrString);
	}

	//****** GET METHODEN *******

	public int getDay() {		// **** get-Methoden fr Int Werte
		attributeUpdate();	// bergibt den Tag als Int zB 28
		return tagInt;
	}
	public int getMonth() {		// bergibt den Monat als Int zB 9
		attributeUpdate();
		return monatInt;
	}
	public int getYear() {		//bergibt das Jahr als Int zB 2001
		attributeUpdate();
		return jahrInt;
	}

	public String getDayText() {	// **** get-Methoden fr String Werte
		attributeUpdate();	//bergibt den Tag als String zB "28"
		return tagString;
	}
	public String getMonthText() {	//bergibt den Monat als String zB "06"
		attributeUpdate();
		return monatString;	
	}
	public String getYearText() {	//bergibt das Jahr als String zB "1999"
		attributeUpdate();
		return jahrString;
	}
	//public String getText() {	//berschrieben !!! Gibt Datum als String
		//attributeUpdate();	// zB "11.12.2000"
		//return tagString + "." + monatString + "." + jahrString;
	//}

	public Calendar getDate() {	// **** get-Methode fr Calendar Objekte
		attributeUpdate();	// bergibt den Inhalt des Textfeldes als Calendar Objekt
		Calendar aktDate = new GregorianCalendar(tagInt, monatInt, jahrInt);
		return aktDate; //hier neues kalenderobjekt erzeugen
	}
	//****** GET METHODEN Ende *

	//****** Zusatz METHODEN ***
	private void attributeUpdate() { // MUSS vor Rckgabe in get-Methoden aufgerufen werden
		tagString = this.getText().substring(0, 2); 	// Achtung: Der Wert kann ungltig
		monatString = this.getText().substring(3, 5);	// sein. Also zB 32 fr den Tag.
		jahrString = this.getText().substring(6);

		tagInt = new Integer(tagString).intValue();	// Getestet! Substring Positionen
		monatInt = new Integer(monatString).intValue();	// stimmen.
		jahrInt = new Integer(jahrString).intValue(); 
	}
	public boolean hasZeroField() {				// Methode, die prft, ob ein Feld
		boolean hasZero=false;				// "00" enthlt.
		attributeUpdate();
		if (tagString.equals("00")) hasZero=true;
		if (monatString.equals("00")) hasZero=true;	// Bei Jahr ist "0000" erlaubt
		return hasZero;				
	}
	public boolean hasNoZeroField() {			// Methode, die prft, ob ein Feld
		boolean hasZero=true;				// "00" enthlt.
		attributeUpdate();
		if (tagString.equals("00")) hasZero=false;
		if (monatString.equals("00")) hasZero=false;	// Bei Jahr ist "0000" erlaubt
		return hasZero;				
	}

	static boolean isDate(int tag, int monat, int jahr) {	// Klassenmethode !!!!!
		//monat++;					//MONATE ZHLEN AB 0 !!!!
		boolean itIsADate=true;
		GregorianCalendar testDate = new GregorianCalendar(jahr, monat, tag);
		for(int i = 0; i < tageMax.length ; i++) {	//schauen ob maximaler tag im
			if(monat==i) {				//monat berschritten ist
				if(tag>tageMax[i]) itIsADate=false;
			}
		}
		if(testDate.isLeapYear(jahr)) {			//bei schaltjahr auch 29 feb.
			if ((monat==1) && (tag==29)) itIsADate=true;
		}
		if(monat>11) itIsADate=false;			//wenn monat grer 12 dann falsch
		if (tag==0) itIsADate=false;			// wenn tag = 0 dann falsch
		if ((monat+1)==0) itIsADate=false;		// wenn monat = 0 dann falsch
		return itIsADate;
	}
	static boolean isDate(String checkString) {		// Klassenmethode !!!!!
		boolean itIsADate=false;
		if(checkString.length()==10) {
			int tag = new Integer(checkString.substring(0, 2)).intValue();
			int monat = new Integer(checkString.substring(3, 5)).intValue()-1;
			int jahr = new Integer(checkString.substring(0, 2)).intValue();
			if(isDate(tag, monat, jahr)==true) itIsADate=true;
		}
		return itIsADate;
	}
}
